/*
This is a part of the IconDesk source code.

Copyright (C) 2003 Erik Christiansson, aka Sci
www.alphafish.com
erik@alphafish.com

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "FileIcon.h"

UINT (WINAPI *_PrivateExtractIcons)(LPCTSTR,int,int,int,HICON*,UINT*,UINT,UINT) = NULL;

IContextMenu2 *g_IContext2 = NULL;
IContextMenu3 *g_IContext3 = NULL;

FileIcon::FileIcon()
{
	if(!_PrivateExtractIcons) {
		// Bind the icon extraction function
		_PrivateExtractIcons = (UINT (WINAPI *)(LPCTSTR,int,int,int,HICON*,UINT*,UINT,UINT)) GetProcAddress(GetModuleHandle("USER32.DLL"), "PrivateExtractIconsA");
		if(!_PrivateExtractIcons) {
			DWORD dw = GetLastError();
			LogPrintf(LOG_ERROR, "Could not bind _PrivateExtractIcons (Error: %u)", dw);
		}
	}
	_pidl = NULL;
}

FileIcon::~FileIcon()
{
	if(_pidl) {
		LPMALLOC pMalloc;
		if(SHGetMalloc(&pMalloc) == NOERROR) {
			pMalloc->Free(_pidl);
			pMalloc->Release();
		}
	}
}

void FileIcon::SetPIDL(LPITEMIDLIST pidl)
{
	_pidl = pidl;
}

void FileIcon::Execute(std::string sCommand)
{
	if(!stricmp(sCommand, ".none")) {
		return;
	}
	else if(!stricmp(sCommand, ".context")) {
		_PopupContextMenu();
		InvalidateRect(hParent, NULL, TRUE);
	}
	else if(!stricmp(sCommand, ".open")) {
		// Why doesn't ShellExecuteEx with NULL as verb work on folders?
		_ExecuteDefaultContextMenuItem();
	}
	else if(!stricmp(sCommand, ".properties")) {
		SHELLEXECUTEINFO sei = {sizeof(sei)};
		sei.fMask = SEE_MASK_INVOKEIDLIST;
		sei.lpVerb = "properties";
		sei.lpIDList = _pidl;
		ShellExecuteEx(&sei);
	}
	else {
		/*std::string sSought = "{filename}";
		std::string sLower = sCommand;
		transform(sLower.begin(), sLower.end(), sLower.begin(), tolower);

		int nPos = sCommand.find(sSought);
		if(nPos != std::string::npos) {
			sCommand.replace(nPos, sSought.size(), sFilename); 
		}*/
		//LSExecute(NULL, sCommand.c_str(), SW_NORMAL);
		ErrorMessage("'%s' is not a recognized command.", sCommand.c_str());
	}
}

/**
 * _ExecuteDefaultContextMenuItem
 *
 * Originally stolen from popup2 by the LS dev-team
 */
void FileIcon::_ExecuteDefaultContextMenuItem()
{
	LPMALLOC pMalloc;
	LPSHELLFOLDER psfFolder, psfNextFolder;
	LPITEMIDLIST pidlMain, pidlItem, pidlNextItem;
	LPCONTEXTMENU pContextMenu;
	CMINVOKECOMMANDINFO ici;
	UINT nCount, nCmd;
	HMENU hMenu;

	pidlMain = PIDLCopy(_pidl);

	// Get current cursor position & window
	POINT point;
	HWND hwnd;

	GetCursorPos(&point);
	hwnd = WindowFromPoint(point);

	// Get pointers to the shell's IMalloc interface and the desktop's
	// IShellFolder interface.
	if(!SUCCEEDED(SHGetMalloc(&pMalloc))) {
		return;
	}

	if(!SUCCEEDED(SHGetDesktopFolder(&psfFolder))) {
		pMalloc->Release();
		return;
	}

	nCount = PIDLGetItemCount(pidlMain);
	if(nCount > 0) {
		// nCount must be > 0
		// Initialize psfFolder with a pointer to the IShellFolder
		// interface of the folder that contains the item whose context
		// menu we're after, and initialize pidlItem with a pointer to
		// the item's item ID. If nCount > 1, this requires us to walk
		// the list of item IDs stored in pidlMain and bind to each
		// subfolder referenced in the list.
		pidlItem = pidlMain;

		while(--nCount) {
			// Create a 1-item item ID list for the next item in pidlMain.
			pidlNextItem = PIDLCopyFirst(pidlItem);
			if(pidlNextItem == NULL) {
				pMalloc->Free(pidlMain);
				psfFolder->Release();
				pMalloc->Release();
				return;
			}

			// Bind to the folder specified in the new item ID list.
			if(!SUCCEEDED(psfFolder->BindToObject(pidlNextItem, NULL, IID_IShellFolder, (void**)&psfNextFolder))) {
				pMalloc->Free(pidlNextItem);
				pMalloc->Free(pidlMain);
				psfFolder->Release();
				pMalloc->Release();
				return;
			}

			// Release the IShellFolder pointer to the parent folder
			// and set psfFolder equal to the IShellFolder pointer for
			// the current folder.
			psfFolder->Release();
			psfFolder = psfNextFolder;

			// Release the storage for the 1-item item ID list we created
			// just a moment ago and initialize pidlItem so that it points
			// to the next item in pidlMain.
			pMalloc->Free(pidlNextItem);
			pidlItem = PIDLGetNextItem(pidlItem);
		}

		// Get a pointer to a IContextMenu object for the item
		if(SUCCEEDED(psfFolder->GetUIObjectOf(NULL, 1, (LPCITEMIDLIST*)&pidlItem, IID_IContextMenu, NULL, (void**)&pContextMenu))) {
			// Create menu
			hMenu = CreatePopupMenu();
			// Let the context menu handler fill the menu
			if(SUCCEEDED(pContextMenu->QueryContextMenu(hMenu, 0, 1, 0x7FFF, CMF_NORMAL))) {
				// Display the context menu.
				//nCmd = TrackPopupMenu(hMenu, TPM_LEFTALIGN | TPM_LEFTBUTTON | TPM_RIGHTBUTTON | TPM_RETURNCMD, point.x, point.y, 0, hwnd, NULL);
				nCmd = GetMenuDefaultItem(hMenu, FALSE, 0);

				// If a command was selected from the menu, execute it.
				if(nCmd) {
					ici.cbSize = sizeof(CMINVOKECOMMANDINFO);
					ici.fMask = SEE_MASK_FLAG_NO_UI;
					ici.hwnd = hwnd;
					ici.lpVerb = MAKEINTRESOURCE(nCmd - 1);
					ici.lpParameters = NULL;
					ici.lpDirectory = NULL;
					ici.nShow = SW_SHOWNORMAL;
					ici.dwHotKey = 0;
					ici.hIcon = NULL;

					pContextMenu->InvokeCommand(&ici);
				}
			}
			DestroyMenu(hMenu);
			pContextMenu->Release();
		}
	}
	pMalloc->Free(pidlMain);

	// Clean up and return.
	psfFolder->Release();
	pMalloc->Release();

	return;
}

/**
 * _PopupContextMenu
 *
 * Originally stolen from popup2 by the LS dev-team
 */
void FileIcon::_PopupContextMenu()
{
	LPMALLOC pMalloc;
	LPSHELLFOLDER psfFolder, psfNextFolder;
	LPITEMIDLIST pidlMain, pidlItem, pidlNextItem;
	LPCONTEXTMENU pContextMenu;
	CMINVOKECOMMANDINFO ici;
	UINT nCount, nCmd;
	HMENU hMenu;

	pidlMain = PIDLCopy(_pidl);

	// Get current cursor position & window
	POINT point;
	HWND hwnd;

	GetCursorPos(&point);
	hwnd = WindowFromPoint(point);

	// Get pointers to the shell's IMalloc interface and the desktop's
	// IShellFolder interface.
	if(!SUCCEEDED(SHGetMalloc(&pMalloc))) {
		return;
	}

	if(!SUCCEEDED(SHGetDesktopFolder(&psfFolder))) {
		pMalloc->Release();
		return;
	}

	nCount = PIDLGetItemCount(pidlMain);
	if(nCount > 0) {
		// nCount must be > 0
		// Initialize psfFolder with a pointer to the IShellFolder
		// interface of the folder that contains the item whose context
		// menu we're after, and initialize pidlItem with a pointer to
		// the item's item ID. If nCount > 1, this requires us to walk
		// the list of item IDs stored in pidlMain and bind to each
		// subfolder referenced in the list.
		pidlItem = pidlMain;

		while(--nCount) {
			// Create a 1-item item ID list for the next item in pidlMain.
			pidlNextItem = PIDLCopyFirst(pidlItem);
			if(pidlNextItem == NULL) {
				pMalloc->Free(pidlMain);
				psfFolder->Release();
				pMalloc->Release();
				return;
			}

			// Bind to the folder specified in the new item ID list.
			if(!SUCCEEDED(psfFolder->BindToObject(pidlNextItem, NULL, IID_IShellFolder, (void**)&psfNextFolder))) {
				pMalloc->Free(pidlNextItem);
				pMalloc->Free(pidlMain);
				psfFolder->Release();
				pMalloc->Release();
				return;
			}

			// Release the IShellFolder pointer to the parent folder
			// and set psfFolder equal to the IShellFolder pointer for
			// the current folder.
			psfFolder->Release();
			psfFolder = psfNextFolder;

			// Release the storage for the 1-item item ID list we created
			// just a moment ago and initialize pidlItem so that it points
			// to the next item in pidlMain.
			pMalloc->Free(pidlNextItem);
			pidlItem = PIDLGetNextItem(pidlItem);
		}

		int iMenuType = 1;
		g_IContext2 = NULL;
		g_IContext3 = NULL;

		// Get a pointer to a IContextMenu object for the item
		if(SUCCEEDED(psfFolder->GetUIObjectOf(NULL, 1, (LPCITEMIDLIST*)&pidlItem, IID_IContextMenu, NULL, (void**)&pContextMenu))) {
			// Attempt to "upgrade" to IContextMenu3/2
			LPCONTEXTMENU pCMX = NULL;
			if(SUCCEEDED(pContextMenu->QueryInterface(IID_IContextMenu3, (void**)&pCMX))) {
				pContextMenu->Release();
				pContextMenu = pCMX;
				iMenuType = 3;
			}
			else if(SUCCEEDED(pContextMenu->QueryInterface(IID_IContextMenu2, (void**)&pCMX))) {
				pContextMenu->Release();
				pContextMenu = pCMX;
				iMenuType = 2;
			}

			// Create menu
			hMenu = CreatePopupMenu();
			// Let the context menu handler fill the menu
			if(SUCCEEDED(pContextMenu->QueryContextMenu(hMenu, 0, 1, 0x7FFF, CMF_NORMAL))) {
				// Subclass if it's version 2 or 3
				WNDPROC OldWndProc = NULL;
				if(iMenuType > 1) {
					OldWndProc = (WNDPROC)SetWindowLong(hwnd, GWL_WNDPROC, (DWORD)_HookWndProc);
					SetProp(hwnd, TEXT("OldWndProc"), OldWndProc);
					if(iMenuType == 2) {
						g_IContext2 = (LPCONTEXTMENU2)pContextMenu;
					}
					else {
						g_IContext3 = (LPCONTEXTMENU3)pContextMenu;
					}
				}

				// Display the context menu.
				nCmd = TrackPopupMenu(hMenu, TPM_LEFTALIGN | TPM_LEFTBUTTON | TPM_RIGHTBUTTON | TPM_RETURNCMD, point.x, point.y, 0, hwnd, NULL);

				// Unsubclass
				if(OldWndProc) {
					SetWindowLong(hwnd, GWL_WNDPROC, (DWORD)OldWndProc);
					RemoveProp(hwnd, TEXT("OldWndProc"));
				}

				// If a command was selected from the menu, execute it.
				if(nCmd) {
					ici.cbSize = sizeof(CMINVOKECOMMANDINFO);
					ici.fMask = SEE_MASK_FLAG_NO_UI;
					ici.hwnd = hwnd;
					ici.lpVerb = MAKEINTRESOURCE(nCmd - 1);
					ici.lpParameters = NULL;
					ici.lpDirectory = NULL;
					ici.nShow = SW_SHOWNORMAL;
					ici.dwHotKey = 0;
					ici.hIcon = NULL;

					pContextMenu->InvokeCommand(&ici);
				}
			}
			DestroyMenu(hMenu);
			pContextMenu->Release();
			g_IContext2 = NULL;
			g_IContext3 = NULL;
		}
	}
	pMalloc->Free(pidlMain);

	// Clean up and return.
	psfFolder->Release();
	pMalloc->Release();

	return;
}

LRESULT CALLBACK FileIcon::_HookWndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch(message)
	{ 
	case WM_MENUCHAR:	// only supported by IContextMenu3
		if(g_IContext3) {
			LRESULT lResult = 0;
			g_IContext3->HandleMenuMsg2(message, wParam, lParam, &lResult);
			return lResult;
		}
		break;

	case WM_DRAWITEM:
	case WM_MEASUREITEM:
		if(wParam) {
			break; // if wParam != 0 then the message is not menu-related
		}
  
	case WM_INITMENUPOPUP:
		if(g_IContext2) {
			g_IContext2->HandleMenuMsg(message, wParam, lParam);
		}
		else {	// version 3
			g_IContext3->HandleMenuMsg(message, wParam, lParam);
		}
		return (message == WM_INITMENUPOPUP ? 0 : TRUE); // inform caller that we handled WM_INITPOPUPMENU by ourself
		break;

	default:
		break;
	}

	// call original WndProc of window to prevent undefined bevhaviour of window
	return ::CallWindowProc((WNDPROC)GetProp(hWnd, TEXT("OldWndProc")), hWnd, message, wParam, lParam);
}

void FileIcon::LoadIcon(int nSize)
{
	SetIcon(_GetIconFromPath(sFilename, nSize));
}

HICON FileIcon::_GetIconFromPath(std::string sPath, int nSize)
{
	if(!_PrivateExtractIcons) {
		return NULL;
	}

	HICON hIcon = NULL;
	std::string sExt;

	if((lFlags & ICON_FOLDER) > 0) {
		if((lFlags & ICON_FILESYSTEM) > 0) {
			sExt = "Folder";
		}
	}
	else {
		char* pszExt = strrchr(sPath.c_str(), '.');
		if(pszExt) {
		    sExt = pszExt;
		}
	}
    
	if(!stricmp(sExt, ".ico") || !stricmp(sExt, ".cur") || !stricmp(sExt, ".ani") || !stricmp(sExt, ".exe")) {
		// Executables, icons and cursors have their own icons
		_PrivateExtractIcons(sPath.c_str(), 0, nSize, nSize, &hIcon, NULL, 1, NULL);
    }
    else if(!stricmp(sExt, ".lnk")) {
		// Special case for link files
		// If the icon is set for the link, retrieve it
		SHFILEINFO shfi;
		SHGetFileInfo(sPath.c_str(), 0, &shfi, sizeof(SHFILEINFO), SHGFI_ICONLOCATION);
		// Extract the icon
		if(_PrivateExtractIcons(shfi.szDisplayName, shfi.iIcon, nSize, nSize, &hIcon, NULL, 1, NULL) == 0xFFFFFFFF) {
			// If the icon was not explicitly set in the linkfile, we'll use the icon from the target file
			HRESULT hRes;				
			char szInfo[1024];
			IShellLink* psl;

			// Create the ShellLink object
			hRes = CoCreateInstance(CLSID_ShellLink, NULL, CLSCTX_INPROC_SERVER, IID_IShellLink, (LPVOID*)&psl);
			if(SUCCEEDED(hRes)) {
				IPersistFile* ppf;
				// Bind the ShellLink object to the Persistent File
				hRes = psl->QueryInterface(IID_IPersistFile, (LPVOID *) &ppf);
				if(SUCCEEDED(hRes)) {
					WORD wsz[MAX_PATH];

					MultiByteToWideChar(CP_ACP, MB_PRECOMPOSED, sPath.c_str(), -1, wsz, MAX_PATH);
					hRes = ppf->Load(wsz, 0);
					
					if(SUCCEEDED(hRes)) {
						// Retrieve the target of the link file
						psl->GetPath(szInfo, 1024, NULL, SLGP_UNCPRIORITY);
					}
				}
			}
			psl->Release();

			// Extract the icon from the link target
			//_PrivateExtractIcons(szInfo, 0, cxDesired, cyDesired, &hIcon, NULL, 1, NULL);
			hIcon = _GetIconFromPath(szInfo, nSize);
		}
    }
    else {
		// Load icons for all other file types
		if(sExt.size() > 0) {
			// Retrieve the icon associated with the file type from the registry
			char szFileType[1024], szIcon[1024];
			int iIconIndex = 0;
			DWORD dwDataSize = 1024;
			// Get the file type from the extension
			if(SHGetValue(HKEY_CLASSES_ROOT, sExt.c_str(), NULL, NULL, szFileType, &dwDataSize) == ERROR_SUCCESS) {
				strcat(szFileType, "\\DefaultIcon");
				dwDataSize = 1024;
				// Get the icon
				if(SHGetValue(HKEY_CLASSES_ROOT, szFileType, NULL, NULL, szIcon, &dwDataSize) == ERROR_SUCCESS) {
					char* pszSplit = strrchr(szIcon, ',');
					if(pszSplit) {
						iIconIndex = atoi(pszSplit + 1);
						(*pszSplit) = '\0';
					}

					// Extract the icon
					_PrivateExtractIcons(szIcon, iIconIndex, nSize, nSize, &hIcon, NULL, 1, NULL);
				}
			}

			if(!hIcon) {
				// No icon
				_PrivateExtractIcons("shell32.dll", 0, nSize, nSize, &hIcon, NULL, 1, NULL);
			}
		}
		else {
			// Let the shell decide the icon
			SHFILEINFO shfi;
			SHGetFileInfo(sPath.c_str(), 0, &shfi, sizeof(SHFILEINFO), SHGFI_ICONLOCATION);
			// Extract the icon
			_PrivateExtractIcons(shfi.szDisplayName, shfi.iIcon, nSize, nSize, &hIcon, NULL, 1, NULL);
		}
    }

	// If all else fails
	if(hIcon == NULL) {
		SHFILEINFO shfi;
		SHGetFileInfo(sPath.c_str(), 0, &shfi, sizeof(SHFILEINFO), SHGFI_ICON | (nSize <= 24 ? SHGFI_SMALLICON : 0));
		return shfi.hIcon;
	}

	return hIcon;
}