/*
This is a part of the IconDesk source code.

Copyright (C) 2003 Erik Christiansson, aka Sci
www.alphafish.com
erik@alphafish.com

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#ifndef __GROUP_H_
#define __GROUP_H_

#include <windows.h>
#include <windowsx.h>
#include <commctrl.h>
#include <shlobj.h>
#include <map>
#include <list>
#include <string>
#include "..\lsapi\lsapi.h"
#include "..\lsapi\lswinbase.h"
#include "pidl.h"
#include "tooltips.h"
#include "log.h"
#include "Icon.h"
#include "FileIcon.h"
#include "ShellNotify.h"
#include "utility.h"
#include "Settings.h"

enum DESKTOPDIRECTION
{
	DD_RIGHT,
	DD_DOWN,
	DD_LEFT,
	DD_UP
};

// Defined effects
#define FX_HUE			0x1
#define FX_SATURATION	0x2
#define FX_ANY			(FX_HUE | FX_SATURATION)

// Defined modification keys
#define ME_LBUTTONUP    0x0001;
#define ME_LBUTTONDOWN  0x0002;
#define ME_LBUTTONDBL   0x0004;
#define ME_MBUTTONUP    0x0008;
#define ME_MBUTTONDOWN  0x0010;
#define ME_MBUTTONDBL   0x0020;
#define ME_RBUTTONUP    0x0040;
#define ME_RBUTTONDOWN  0x0080;
#define ME_RBUTTONDBL   0x0100;
#define ME_CONTROL      0x0200;
#define ME_MENU         0x0400;
#define ME_SHIFT        0x0800;
#define ME_APPS         0x2000;

class ShellNotify;

class Group
{
private:
	typedef std::list<Icon*> IconList;
	typedef IconList::iterator IconListIter;

	typedef std::list<std::string> StringList;
	typedef StringList::iterator StringListIter;

	typedef std::list<ShellNotify*> ShellNotifyList;
	typedef ShellNotifyList::iterator ShellNotifyListIter;

	typedef std::map<UINT, std::string> MouseMap;
	typedef MouseMap::iterator MouseMapIter;
	typedef MouseMap::value_type MouseMapValue;

	class CompareIconFlags
	{
	private:
		long _lFlags;
		bool _bReverse;

	public:
		CompareIconFlags(long lFlags, bool bReverse = false)
		{
			_lFlags = lFlags;
			_bReverse = bReverse;
		}

		bool operator()(Icon* i1, Icon* i2)
		{
			if(_lFlags == 0) {
				if(_bReverse) {
					return (stricmp(i1->sTitle, i2->sTitle) > 0);
				}
				else {
					return (stricmp(i1->sTitle, i2->sTitle) < 0);
				}
			}
			else {
				if(_bReverse) {
					return ((i1->lFlags & _lFlags) < (i2->lFlags & _lFlags));
				}
				else {
					return ((i1->lFlags & _lFlags) > (i2->lFlags & _lFlags));
				}
			}
		}

	};

public:
	Group(std::string sName, Settings* pSettings, HWND parentWnd);
	~Group();

	void show();
	void hide();
	void toggle();

	void paintIcons(HDC hDC);
	void update();
	BOOL mouseEvent(UINT uMsg, WPARAM wParam, LPARAM lParam);
	
	std::string sName;

private:
	void _loadIcons();
	void _loadFolder(std::string sFolder);
	void _deleteIcons();

	void _loadTooltips();
	void _deleteTooltips();

	Icon* _getIconAtPt(int x, int y);

	void _StrRetToBuf(LPSTRRET pstr, LPCITEMIDLIST pidl, LPTSTR pszBuf, UINT cchBuf);

	Settings* _pSettings;

	HWND _parentWnd;
	bool _visible;

	IconList _iconList;
	bool _iconsLoaded;
	Icon* _selectedIcon;

	int _iconX;
	int _iconY;
	int _iconSize;
	int _iconSpacingX;
	int _iconSpacingY;

	StringList _folders;
	ShellNotifyList _notifiers;

	StringList _patternHide;
	StringList _patternFirst;
	StringList _patternLast;

	MouseMap _mouseEvents;

	bool _iconFoldersFirst;
	bool _iconNonFilesystemFirst;
	bool _iconHideExtension;
	bool _iconShowHidden;
	bool _hideText;
	bool _showTooltips;

	DESKTOPDIRECTION _iconDirection;
	DESKTOPDIRECTION _iconWrapDirection;
	int _iconWrapCount;

	int _textSpacing;
	int _textHeight;
	int _textWidth;

	HFONT _textFont;
	COLORREF _textColor;
	HBRUSH _textBackground;
	UINT _textAlign;
	DESKTOPDIRECTION _textPosition;

	COLORREF _textShadowColor;
	int _textShadowDepth;

	COLORREF _textOutlineColor;
	bool _textOutline;

	// Icon effects
	COLORREF _hueColor;
	unsigned char _hueIntensity;
	unsigned char _saturation;
	unsigned char _effectFlags;

};

#endif//__GROUP_H_
