/*
This is a part of the IconDesk source code.

Copyright (C) 2003 Erik Christiansson, aka Sci
www.alphafish.com
erik@alphafish.com

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#ifndef __SETTINGS_H_
#define __SETTINGS_H_

#include <string>
#include <iterator>

class Settings {
public:
	class Enum {
	public:
		virtual bool Next()
		{
			return false;
		}

		virtual std::string GetString()
		{
			return NULL;
		}

	};

	class iterator : public std::iterator<std::input_iterator_tag, std::string>
	{
	private:
	public:
		bool			_bEnd;
		Settings::Enum*	_pEnum;

		iterator() : _bEnd(true), _pEnum(NULL)
		{
		}

		iterator(Enum* pEnum) : _bEnd(false), _pEnum(pEnum)
		{
			if(_pEnum == NULL) {
				_bEnd = true;
			}
			else {
				operator++();
			}
		}

		iterator(std::string) : _bEnd(true), _pEnum(NULL)
		{
		}

		iterator(const iterator& it) : _bEnd(it._bEnd), _pEnum(it._pEnum)
		{
		}

		~iterator()
		{
			if(_pEnum) {
				delete _pEnum;
			}
		}

		iterator& operator++()
		{
			if(_pEnum) {
				_bEnd = !_pEnum->Next();
			}
			return *this;
		}

		bool operator==(const iterator& it) const
		{
			return _bEnd == it._bEnd;
		}

		bool operator!=(const iterator& it) const
		{
			return !(*this == it);
		}

		std::string operator*()
		{
			if(_pEnum != NULL) {
				return _pEnum->GetString();
			}
			else {
				return "";
			}
		}
	};

	Settings()
	{
		_end = iterator();
	}

	virtual std::string GetString(std::string sKey, std::string sDefault)
	{
		return sDefault;
	}

	virtual bool GetBool(std::string sKey, bool bDefault)
	{
		return bDefault;
	}

	virtual int GetInt(std::string sKey, int nDefault)
	{
		return nDefault;
	}

	virtual COLORREF GetColor(std::string sKey, COLORREF cDefault)
	{
		return cDefault;
	}

	virtual Settings::Enum* GetEnum(std::string sKey, bool bSpecial = false)
	{
		return new Enum();
	}

	virtual Settings* GetChild(std::string sKey)
	{
		return new Settings();
	}

	virtual Settings::iterator end()
	{
		return _end;
	}

	virtual Settings::iterator begin(std::string sKey, bool bSpecial = false)
	{
		return iterator();
	}

private:
	iterator _end;

};

#endif//__SETTINGS_H_