/*
This is a part of the IconDesk source code.

Copyright (C) 2003 Erik Christiansson, aka Sci
www.alphafish.com
erik@alphafish.com

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#ifndef __SETTINGSLITESTEP_H_
#define __SETTINGSLITESTEP_H_

#include <string>
#include <stdio.h>
#include "Settings.h"
//#include "../lsapi/lsapi.h"

int(*_GetRCInt)(LPCSTR lpKeyName, int nDefault);
BOOL(*_GetRCString)(LPCSTR lpKeyName, LPSTR value, LPCSTR defStr, int maxLen);
BOOL(*_GetRCBoolDef)(LPCSTR lpKeyName, BOOL bDefault);
COLORREF(*_GetRCColor)(LPCSTR lpKeyName, COLORREF colDef);
FILE*(*_LCOpen)(LPCSTR szPath);
BOOL(*_LCClose)(FILE *f);
BOOL(*_LCReadNextConfig)(FILE *f, LPCSTR szPrefix, LPSTR szBuffer, DWORD dwLength);
int(*_LCTokenize)(LPCSTR szString, LPSTR* lpszBuffers, DWORD dwNumBuffers, LPSTR szExtraParameters);

#define MAX_RCCOMMAND              64
#define MAX_LINE_LENGTH            4096

class SettingsLitestep : public Settings {
public:
	class EnumLitestep : public Settings::Enum {
	public:
		EnumLitestep(std::string sEnumKey)
		{
			_fpStep = _LCOpen(NULL);
			_sEnumKey = sEnumKey;
		}

		~EnumLitestep()
		{
			if(_fpStep) {
				_LCClose(_fpStep);
			}
		}

		bool Next()
		{
			char szBuffer[MAX_LINE_LENGTH], szDummy[MAX_RCCOMMAND], szData[MAX_LINE_LENGTH];
			char* rszTokens[1] = {szDummy};
			szBuffer[0] = szDummy[0] = szData[0] = 0;
			if(_LCReadNextConfig(_fpStep, _sEnumKey.c_str(), szBuffer, sizeof(szBuffer))) {
				_LCTokenize(szBuffer, rszTokens, 1, szData);
				_sData = szData;
				return true;
			}
			else {
				return false;
			}
		}

		std::string GetString()
		{
			return _sData;
		}

	private:
		FILE* _fpStep;
		std::string _sEnumKey;
		std::string _sData;

	};

	SettingsLitestep(std::string sPrefix) {
		_sPrefix = sPrefix;
	}

	~SettingsLitestep() {
	}

	std::string GetString(std::string sKey, std::string sDefault)
	{
		char szKey[MAX_RCCOMMAND] = "";
		char szBuffer[MAX_LINE_LENGTH] = "";
		sprintf(szKey, "%s%s", _sPrefix.c_str(), sKey.c_str());
		_GetRCString(szKey, szBuffer, sDefault.c_str(), MAX_LINE_LENGTH);
		return szBuffer;
	}

	int GetInt(std::string sKey, int nDefault)
	{
		char szKey[MAX_RCCOMMAND] = "";
		sprintf(szKey, "%s%s", _sPrefix.c_str(), sKey.c_str());
		return _GetRCInt(szKey, nDefault);
	}

	bool GetBool(std::string sKey, bool bDefault)
	{
		char szKey[MAX_RCCOMMAND] = "";
		sprintf(szKey, "%s%s", _sPrefix.c_str(), sKey.c_str());
		return (_GetRCBoolDef(szKey, bDefault) == TRUE);
	}

	COLORREF GetColor(std::string sKey, COLORREF cDefault)
	{
		char szKey[MAX_RCCOMMAND] = "";
		sprintf(szKey, "%s%s", _sPrefix.c_str(), sKey.c_str());
		return _GetRCColor(szKey, cDefault);
	}

	Settings::Enum* GetEnum(std::string sKey, bool bSpecial = false)
	{
		std::string sEnumKey = _sPrefix + sKey;
		if(!bSpecial) {
			sEnumKey = "*" + sEnumKey;
		}

		return new SettingsLitestep::EnumLitestep(sEnumKey);
	}

	Settings* GetChild(std::string sKey)
	{
		return new SettingsLitestep(_sPrefix + sKey);
	}

	Settings::iterator begin(std::string sKey, bool bSpecial = false)
	{
		return iterator(GetEnum(sKey, bSpecial));
	}

private:
	std::string _sPrefix;

};

#endif//__SETTINGSLITESTEP_H_