/*
This is a part of the IconDesk source code.

Copyright (C) 2003 Erik Christiansson, aka Sci
www.alphafish.com
erik@alphafish.com

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#ifndef __SETTINGSREGISTRY_H_
#define __SETTINGSREGISTRY_H_

#include <windows.h>
#include <string>
#include "Settings.h"

class SettingsRegistry : public Settings {
public:
	class EnumRegistry : public Settings::Enum {
	public:
		EnumRegistry(HKEY hKey)
		{
			_hKey = hKey;
			_dwIndex = 0;
		}

		~EnumRegistry()
		{
			if(_hKey) {
				RegCloseKey(_hKey);
			}
		}

		bool Next()
		{
			TCHAR szName[1024];
			DWORD dwNameSize = 1024;
			char szData[1024];
			DWORD dwDataSize = 1024;
			DWORD dwType = NULL;

			while(dwType != REG_SZ) {
				if(RegEnumValue(_hKey, _dwIndex, szName, &dwNameSize, NULL, &dwType, (BYTE*)szData, &dwDataSize) == ERROR_NO_MORE_ITEMS) {
					return false;
				}
				_dwIndex++;
			}
			
			_sData = szData;
			return true;
		}

		std::string GetString()
		{
			return _sData;
		}

	private:
		HKEY _hKey;
		DWORD _dwIndex;
		std::string _sData;

	};

	SettingsRegistry(HKEY hMainKey, std::string sSubKey) {
		//if(RegCreateKeyEx(hMainKey, sSubKey.c_str(), 0, NULL, REG_OPTION_NON_VOLATILE, KEY_ENUMERATE_SUB_KEYS | KEY_READ | KEY_QUERY_VALUE, NULL, &_hKey, NULL) != ERROR_SUCCESS) {
		if(RegOpenKeyEx(hMainKey, sSubKey.c_str(), NULL, KEY_ENUMERATE_SUB_KEYS | KEY_READ | KEY_QUERY_VALUE, &_hKey) != ERROR_SUCCESS) {
			_hKey = NULL;
			// Do error handling
		}
	}

	~SettingsRegistry() {
		if(_hKey != NULL) {
			RegCloseKey(_hKey);
		}
	}

	std::string GetString(std::string sKey, std::string sDefault)
	{
		DWORD dwType = 0;
		char szData[1024];
		DWORD dwDataSize = 1024;
		if(RegQueryValueEx(_hKey, sKey.c_str(), NULL, &dwType, (BYTE*)&szData, &dwDataSize) != ERROR_SUCCESS) {
			return sDefault;
		}
		else if(dwType != REG_SZ) {
			return sDefault;
		}
		else {
			return szData;
		}
	}

	int GetInt(std::string sKey, int nDefault)
	{
		DWORD dwType = 0;
		int nData = 0;
		DWORD dwDataSize = sizeof(int);
		if(RegQueryValueEx(_hKey, sKey.c_str(), NULL, &dwType, (BYTE*)&nData, &dwDataSize) != ERROR_SUCCESS) {
			return nDefault;
		}
		else if(dwType != REG_DWORD) {
			return nDefault;
		}
		else {
			return nData;
		}
	}

	bool GetBool(std::string sKey, bool bDefault)
	{
		DWORD dwType = 0;
		int nData = 0;
		DWORD dwDataSize = sizeof(int);
		if(RegQueryValueEx(_hKey, sKey.c_str(), NULL, &dwType, (BYTE*)&nData, &dwDataSize) != ERROR_SUCCESS) {
			return bDefault;
		}
		else if(dwType != REG_DWORD) {
			return bDefault;
		}
		else {
			return (nData != 0);
		}
	}

	COLORREF GetColor(std::string sKey, COLORREF cDefault)
	{
		DWORD dwType = 0;
		char szData[1024];
		DWORD dwDataSize = 1024;
		if(RegQueryValueEx(_hKey, sKey.c_str(), NULL, &dwType, (BYTE*)&szData, &dwDataSize) != ERROR_SUCCESS) {
			return cDefault;
		}
		else if(dwType != REG_SZ) {
			return cDefault;
		}
		else {
			COLORREF cReturn = strtol(szData, NULL, 16);
            // convert from BGR to RGB
            cReturn = RGB(GetBValue(cReturn), GetGValue(cReturn), GetRValue(cReturn));
			return cReturn;
		}
	}

	Settings::Enum* GetEnum(std::string sKey, bool bSpecial = false)
	{
		HKEY hEnumKey = NULL;
		//if(RegCreateKeyEx(_hKey, sKey.c_str(), 0, NULL, REG_OPTION_NON_VOLATILE, KEY_ENUMERATE_SUB_KEYS | KEY_READ | KEY_QUERY_VALUE, NULL, &hEnumKey, NULL) != ERROR_SUCCESS) {
		if(RegOpenKeyEx(_hKey, sKey.c_str(), NULL, KEY_ENUMERATE_SUB_KEYS | KEY_READ | KEY_QUERY_VALUE, &hEnumKey) != ERROR_SUCCESS) {
			return NULL;
		}
		else {
			return new SettingsRegistry::EnumRegistry(hEnumKey);
		}
	}

	Settings* GetChild(std::string sKey)
	{
		return new SettingsRegistry(_hKey, sKey);
	}

	Settings::iterator begin(std::string sKey, bool bSpecial = false)
	{
		return iterator(GetEnum(sKey, bSpecial));
	}

private:
	HKEY _hKey;

};

#endif//__SETTINGSREGISTRY_H_