/*
This is a part of the IconDesk source code.

Copyright (C) 2003 Erik Christiansson, aka Sci
www.alphafish.com
erik@alphafish.com

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "ShellNotify.h"

ShellNotify::ShellNotify(LPITEMIDLIST pidl, Group* pGroup) : Window("IconDeskShellNotify")
{
	// The folder to watch
	_pidl = pidl;

	// The group to notify
	_pGroup = pGroup;

	// Get some window handles
	_hDesktop = FindWindow("DesktopBackgroundClass", NULL);

	// Bind ShellNotify functions
	SHChangeNotifyRegister = (ULONG (WINAPI*)(HWND,int,LONG,UINT,int,SHChangeNotifyEntry*)) GetProcAddress(GetModuleHandle("SHELL32.DLL"), MAKEINTRESOURCE(2));
	if(!SHChangeNotifyRegister) {
		DWORD dw = GetLastError();
		LogPrintf(LOG_ERROR, "Could not bind SHChangeNotifyRegister (Error: %u)", dw);
	}
	SHChangeNotifyDeregister = (BOOL (WINAPI*)(ULONG)) GetProcAddress(GetModuleHandle("SHELL32.DLL"), MAKEINTRESOURCE(4));
	if(!SHChangeNotifyDeregister) {
		DWORD dw = GetLastError();
		LogPrintf(LOG_ERROR, "Could not bind SHChangeNotifyDeregister (Error: %u)", dw);
	}
	SHChangeNotification_Lock = (HANDLE (WINAPI *)(HANDLE,DWORD,LPITEMIDLIST**,LONG*)) GetProcAddress(GetModuleHandle("SHELL32.DLL"), MAKEINTRESOURCE(644));
	if(!SHChangeNotification_Lock) {
		DWORD dw = GetLastError();
		LogPrintf(LOG_ERROR, "Could not bind SHChangeNotification_Lock (Error: %u)", dw);
	}
	SHChangeNotification_Unlock = (BOOL (WINAPI *)(HANDLE)) GetProcAddress(GetModuleHandle("SHELL32.DLL"), MAKEINTRESOURCE(645));
	if(!SHChangeNotification_Unlock) {
		DWORD dw = GetLastError();
		LogPrintf(LOG_ERROR, "Could not bind SHChangeNotification_Unlock (Error: %u)", dw);
	}

	// Create the window
	if(!createWindow(WS_EX_TOOLWINDOW, "IconDeskShellNotify", WS_CHILD, 0, 0, 0, 0, _hDesktop)) {
		ErrorMessage("Error creating shell notification window.");
		return;
	}
}

ShellNotify::~ShellNotify()
{
	// Destroy the window
	destroyWindow();
}

void ShellNotify::windowProc(Message& message)
{
	BEGIN_MESSAGEPROC
		MESSAGE(onCreate, WM_CREATE)
		MESSAGE(onDestroy, WM_DESTROY)
		MESSAGE(onShellNotify, WM_APP_SHELLNOTIFY)
	END_MESSAGEPROC
}

void ShellNotify::onCreate(Message& message)
{
	// Hide from VWM
	SetWindowLong(hWnd, GWL_USERDATA, 0x49474541);

	SHChangeNotifyEntry lpItems[1];
	lpItems[0].pidl = PIDLCopy(_pidl);
	lpItems[0].fRecursive = FALSE;
	_hNotify = SHChangeNotifyRegister(hWnd, SHCNF_INTERRUPT | SHCNF_SHELL | SHCNF_NO_PROXY, SHCNE_ASSOCCHANGED | SHCNE_ATTRIBUTES | SHCNE_CREATE | SHCNE_DELETE | SHCNE_DRIVEADD | SHCNE_DRIVEREMOVED | SHCNE_MEDIAINSERTED | SHCNE_MEDIAREMOVED | SHCNE_MKDIR | SHCNE_NETSHARE | SHCNE_NETUNSHARE | SHCNE_RENAMEFOLDER | SHCNE_RENAMEITEM | SHCNE_RMDIR | SHCNE_UPDATEDIR | SHCNE_UPDATEIMAGE | SHCNE_INTERRUPT, WM_APP_SHELLNOTIFY, 1, &lpItems[0]);
	if(!_hNotify) {
		Log(LOG_DEBUG, "Could not register shell notifications");
	}
}

void ShellNotify::onDestroy(Message& message)
{
	if(_hNotify) {
		SHChangeNotifyDeregister(_hNotify);
	}
}

void ShellNotify::onShellNotify(Message& message)
{
	_pGroup->update();
}
