/*
This is a part of the IconDesk source code.

Copyright (C) 2003 Erik Christiansson, aka Sci
www.alphafish.com
erik@alphafish.com

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "main.h"

int initModuleEx(HWND hParentWnd, HINSTANCE hInstance, LPCSTR szPath)
{
	// Save the instance handle
	hInst = hInstance;

	// Bind Litestep logging function
	_LSLog = (BOOL (WINAPI*)(int,LPCSTR,LPCSTR))GetProcAddress(GetModuleHandle("LSAPI.DLL"), "_LSLog@12");
	if(!_LSLog) {
		// The export name might be fixed in the future
		_LSLog = (BOOL (WINAPI*)(int,LPCSTR,LPCSTR))GetProcAddress(GetModuleHandle("LSAPI.DLL"), "LSLog");
	}

	// Bind the Litestep setting functions
	_GetRCInt = (int(*)(LPCSTR,int))GetProcAddress(GetModuleHandle("LSAPI.DLL"), "GetRCInt");
	_GetRCString = (BOOL(*)(LPCSTR,LPSTR,LPCSTR,int))GetProcAddress(GetModuleHandle("LSAPI.DLL"), "GetRCString");
	_GetRCBoolDef = (BOOL(*)(LPCSTR,BOOL))GetProcAddress(GetModuleHandle("LSAPI.DLL"), "GetRCBoolDef");
	_GetRCColor = (COLORREF(*)(LPCSTR,COLORREF))GetProcAddress(GetModuleHandle("LSAPI.DLL"), "GetRCColor");
	_LCOpen = (FILE*(*)(LPCSTR))GetProcAddress(GetModuleHandle("LSAPI.DLL"), "LCOpen");
	_LCClose = (BOOL(*)(FILE*))GetProcAddress(GetModuleHandle("LSAPI.DLL"), "LCClose");
	_LCReadNextConfig = (BOOL(*)(FILE*,LPCSTR,LPSTR,DWORD))GetProcAddress(GetModuleHandle("LSAPI.DLL"), "LCReadNextConfig");
	_LCTokenize = (int(*)(LPCSTR,LPSTR*,DWORD,LPSTR))GetProcAddress(GetModuleHandle("LSAPI.DLL"), "LCTokenize");

	// Register bang commands
	typedef void (_BangCommand)(HWND sender, LPCSTR args);
	BOOL(*_AddBangCommand)(LPCSTR,_BangCommand) = (BOOL(*)(LPCSTR,_BangCommand))GetProcAddress(GetModuleHandle("LSAPI.DLL"), "AddBangCommand");
	if(_AddBangCommand != NULL) {
		_AddBangCommand("!IconDeskShow", bangShow);
		_AddBangCommand("!IconDeskHide", bangHide);
		_AddBangCommand("!IconDeskToggle", bangToggle);
		_AddBangCommand("!IconDeskUpdate", bangUpdate);
	}

	// Set up the settings reader
	pSettings = new SettingsLitestep("IconDesk");

	if(pSettings == NULL) {
		ErrorMessage("Could not create settings reader.");
		return 1;
	}

	// Init the module
	if(initIconDesk() == true) {
		return 0;
	}
	else {
		return 1;
	}
}

void quitModule(HINSTANCE hInstance)
{
	// Remove the bangs
	BOOL(*_RemoveBangCommand)(LPCSTR) = (BOOL(*)(LPCSTR))GetProcAddress(GetModuleHandle("LSAPI.DLL"), "RemoveBangCommand");
	if(_RemoveBangCommand != NULL) {
		_RemoveBangCommand("!IconDeskShow");
		_RemoveBangCommand("!IconDeskHide");
		_RemoveBangCommand("!IconDeskToggle");
		_RemoveBangCommand("!IconDeskUpdate");
	}

	// Quit the module
	quitIconDesk();
}

bool initIconDesk()
{
	// Init the window class
	Window::init(hInst);

	// Get the desktop window
	desktopWnd = FindWindow("DesktopBackgroundClass", NULL);

	// If the desktop window isn't found, wait a while and try again
	if(!desktopWnd) {
		Sleep(500);
		desktopWnd = FindWindow("DesktopBackgroundClass", NULL);
		if(!desktopWnd) {
			Log(LOG_ERROR, "Could not find desktop window");
			ErrorMessage("Could not find desktop window.");
			return false;
		}
	}

	// Subclass the desktop
	oldWndProc = (WNDPROC)GetWindowLong(desktopWnd, GWL_WNDPROC);
	SetWindowLong(desktopWnd, GWL_WNDPROC, (LONG)wndProc);

	// Modify the desktop style to support double clicks
	oldClassStyle = GetClassLong(desktopWnd, GCL_STYLE);
	SetClassLong(desktopWnd, GCL_STYLE, GetClassLong(desktopWnd, GCL_STYLE) | CS_DBLCLKS);

	// Set up tooltop window
	tooltipWnd = CreateWindow(TOOLTIPS_CLASS, NULL, TTS_ALWAYSTIP, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, NULL, NULL, hInst, NULL);
	if(tooltipWnd) {
		SetWindowPos(tooltipWnd, HWND_TOPMOST, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE | SWP_NOACTIVATE);
	}
	tooltipCounter = 0;

	// Read the config
	loadGroups();

	// Redraw the desktop
	InvalidateRect(desktopWnd, NULL, TRUE);

	return true;
}

void quitIconDesk()
{
	// Delete the groups
	deleteGroups();

	// Get rid of the tooltips
	if(tooltipWnd) {
		DestroyWindow(tooltipWnd);
	}

	// Restore the desktop
	SetWindowLong(desktopWnd, GWL_WNDPROC, (LONG)oldWndProc);
	SetClassLong(desktopWnd, GCL_STYLE, oldClassStyle);

	// Redraw the desktop
	InvalidateRect(desktopWnd, NULL, TRUE);
}

LRESULT wndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch(uMsg) {
		case WM_LBUTTONDOWN:
		case WM_RBUTTONDOWN:
		case WM_MBUTTONDOWN:
		case WM_LBUTTONUP:
		case WM_RBUTTONUP:
		case WM_MBUTTONUP:
		case WM_LBUTTONDBLCLK:
		case WM_RBUTTONDBLCLK:
		case WM_MBUTTONDBLCLK:
			{
				// Forward the message to all groups
				BOOL hit = FALSE;
				for(GroupListIter it = groups.begin(); it != groups.end(); it++) {
					if((*it)->mouseEvent(uMsg, wParam, lParam)) {
						hit = TRUE;
					}
				}
				// Only forward the message if we didn't handle it
				if(hit) {
					return 0;
				}
			}
			break;

		case WM_ERASEBKGND:
			// Paint the desktop (if we let the desktop module do it, it'll flicker)
			PaintDesktop((HDC)wParam);

		case WM_PAINT:
			{
				PAINTSTRUCT	ps;
				HDC			hDC;

				if(!wParam) {
					hDC = BeginPaint(hWnd, &ps);
				}
				else {
					hDC = (HDC)wParam;
				}
				
				for(GroupListIter it = groups.begin(); it != groups.end(); it++) {
					(*it)->paintIcons(hDC);
				}

				if(!wParam) {
					EndPaint(hWnd, &ps);
				}
			}
			if(uMsg == WM_ERASEBKGND) {
				return 1;
			}
			else if(uMsg == WM_PAINT) {
				return 0;
			}

	}
	return CallWindowProc(oldWndProc, hWnd, uMsg, wParam, lParam);
}

void bangShow(HWND caller, LPCSTR args)
{
	Group* group = getGroup(args);
	if(group) {
		group->show();
	}
}

void bangHide(HWND caller, LPCSTR args)
{
	Group* group = getGroup(args);
	if(group) {
		group->hide();
	}
}

void bangToggle(HWND caller, LPCSTR args)
{
	Group* group = getGroup(args);
	if(group) {
		group->toggle();
	}
}

void bangUpdate(HWND caller, LPCSTR args)
{
	if(strlen(args)) {
		Group* group = getGroup(args);
		if(group) {
			group->update();
		}
	}
	else {
		// If no group was specified, update all groups
		for(GroupListIter it = groups.begin(); it != groups.end(); it++) {
			(*it)->update();
		}
	}
}

void loadGroups()
{
	for(Settings::iterator it = pSettings->begin("Groups", true); it != pSettings->end(); ++it) {
		std::string sTokens = (*it);
		std::string sBlank = " \t,";
		std::string sToken;
		unsigned int nStart = 0, nLen = 0;

		nStart = 0;
		while(true) {
			nStart = sTokens.find_first_not_of(sBlank, nStart);
			if(nStart == std::string::npos) {
				break;
			}
			nLen = sTokens.find_first_of(sBlank, nStart);
			if(nLen == std::string::npos) {
				nLen = sTokens.size();
			}
			nLen -= nStart;
			if(nLen == 0) {
				break;
			}
			sToken = sTokens.substr(nStart, nLen);
			groups.push_back(new Group(sToken, pSettings->GetChild(sToken), desktopWnd));
			nStart += nLen;
		}
	}
	
	if(groups.size() == 0) {
		groups.push_back(new Group("Default", pSettings->GetChild("Default"), desktopWnd));
	}
}

void deleteGroups()
{
	for(GroupListIter it = groups.begin(); it != groups.end(); it++) {
		delete *it;
	}
	groups.clear();
}

Group* getGroup(std::string sName)
{
	for(GroupListIter it = groups.begin(); it != groups.end(); it++) {
		if(!strcmpi((*it)->sName.c_str(), sName.c_str())) {
			return *it;
		}
	}
	return NULL;
}

UINT AddTooltip(std::string sText, RECT rect, UINT id)
{
	TOOLINFO ti;

	if(!tooltipWnd) {
		return 0;
	}

	if(id == 0) {
		id = ++tooltipCounter;
	}

	ti.cbSize = sizeof(TOOLINFO);
	ti.uFlags = TTF_SUBCLASS;
	ti.hwnd = desktopWnd;
	ti.uId = id;
	ti.rect = rect;
	ti.hinst = hInst;
	ti.lpszText = (LPSTR)sText.c_str();
	ti.lParam = 0;

	SendMessage(tooltipWnd, TTM_ADDTOOL, 0, LPARAM(&ti));

	return id;
}

void DeleteTooltip(UINT id)
{
	TOOLINFO ti;
	RECT emptyRect = {0, 0, 0, 0};

	if(!tooltipWnd) {
		return;
	}

	ti.cbSize = sizeof(TOOLINFO);
	ti.uFlags = 0;
	ti.hwnd = desktopWnd;
	ti.uId = id;
	ti.rect = emptyRect;
	ti.hinst = hInst;
	ti.lpszText = NULL;
	ti.lParam = 0;

	SendMessage(tooltipWnd, TTM_DELTOOL, 0, LPARAM(&ti));
}

BOOL Log(int nLevel, LPCSTR pszMessage)
{
	if(_LSLog) {
		return _LSLog(nLevel, "IconDesk", pszMessage);
	}
	else {
		if(nLevel == LOG_ERROR) {
			MessageBox(NULL, pszMessage, "IconDesk", MB_SETFOREGROUND);
		}
		return TRUE;
	}
}

BOOL LogPrintf(int nLevel, LPCSTR pszFormat, ...)
{
    char szMessage[MAX_LINE_LENGTH];
    va_list argList;
    
    va_start(argList, pszFormat);
    wvsprintf(szMessage, pszFormat, argList);
    va_end(argList);
    
    return Log(nLevel, szMessage);
}
